use std::{
    sync::{atomic::AtomicUsize, Arc},
    thread::{sleep, JoinHandle},
    time::Duration,
};

use rand::Rng;

use crate::stations_list::StationsList;

pub struct StationsThread {
    #[allow(dead_code)]
    min_stations: Arc<AtomicUsize>,
    #[allow(dead_code)]
    join_handle: JoinHandle<()>,
}
impl StationsThread {
    #[allow(dead_code)]
    pub fn turbo(&self, min_stations: usize) {
        self.min_stations
            .store(min_stations, std::sync::atomic::Ordering::Relaxed);
    }
}

pub fn spawn(stations: Arc<StationsList>, min_stations: usize) -> StationsThread {
    let min_stations = Arc::new(AtomicUsize::new(min_stations));
    let min_station_count = Arc::clone(&min_stations);
    let join_handle = std::thread::spawn(move || loop {
        let turbo =
            stations.station_count() < min_station_count.load(std::sync::atomic::Ordering::Relaxed);
        sleep(Duration::from_secs(
            rand::thread_rng().gen_range(10..=80) * if turbo { 1 } else { 60 },
        ));
        match stations.query_for_new_stations() {
            Ok(()) => (),
            Err(e) => {
                eprintln!("Failed to query for new stations:\n{e}");
            }
        }
        for _ in 0..10 {
            sleep(Duration::from_secs(if turbo {
                rand::thread_rng().gen_range(10..=30)
            } else {
                rand::thread_rng().gen_range(30..=120)
            }));
            match stations.requery_random_station(100) {
                Ok(()) => (),
                Err(e) => {
                    eprintln!("Failed to requery stations:\n{e}");
                }
            }
        }
    });
    StationsThread {
        min_stations,
        join_handle,
    }
}
